@@ -1361,8 +1361,6 @@ set_info_from_stat (GVfsBackendSmb *backend,
 		    GFileAttributeMatcher *matcher)
 {
   GFileType file_type;
-  GTimeVal t;
-  char *content_type;
   char *display_name;
 
   if (basename)
@@ -1425,16 +1423,14 @@ set_info_from_stat (GVfsBackendSmb *backend,
                                     G_FILE_ATTRIBUTE_STANDARD_ALLOCATED_SIZE,
                                     statbuf->st_blocks * G_GUINT64_CONSTANT (512));
 
-  t.tv_sec = statbuf->st_mtime;
+  g_file_info_set_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED, statbuf->st_mtime);
 #if defined (HAVE_STRUCT_STAT_ST_MTIMENSEC)
-  t.tv_usec = statbuf->st_mtimensec / 1000;
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, statbuf->st_mtimensec / 1000);
 #elif defined (HAVE_STRUCT_STAT_ST_MTIM_TV_NSEC)
-  t.tv_usec = statbuf->st_mtim.tv_nsec / 1000;
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, statbuf->st_mtim.tv_nsec / 1000);
 #else
-  t.tv_usec = 0;
+  g_file_info_set_attribute_uint32 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, 0);
 #endif
-  g_file_info_set_modification_time (info, &t);
-
 
   if (g_file_attribute_matcher_matches (matcher,
 					G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE) ||
@@ -1445,9 +1441,9 @@ set_info_from_stat (GVfsBackendSmb *backend,
     {
       GIcon *icon = NULL;
       GIcon *symbolic_icon = NULL;
+      char *content_type = NULL;
+      gboolean uncertain_content_type = FALSE;
 
-      content_type = NULL;
-      
       if (S_ISDIR(statbuf->st_mode))
 	{
 	  content_type = g_strdup ("inode/directory");
@@ -1464,7 +1460,7 @@ set_info_from_stat (GVfsBackendSmb *backend,
 	}
       else if (basename != NULL)
 	{
-	  content_type = g_content_type_guess (basename, NULL, 0, NULL);
+	  content_type = g_content_type_guess (basename, NULL, 0, &uncertain_content_type);
 	  if (content_type)
             {
               icon = g_content_type_get_icon (content_type);
@@ -1474,7 +1470,12 @@ set_info_from_stat (GVfsBackendSmb *backend,
       
       if (content_type)
 	{
-	  g_file_info_set_content_type (info, content_type);
+	  if (!uncertain_content_type)
+	    g_file_info_set_content_type (info, content_type);
+
+	  g_file_info_set_attribute_string (info,
+					    G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE,
+					    content_type);
 	  g_free (content_type);
 	}
 
